/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */
package cz.xelfi.quoridor.freemarkerdor;

import java.util.List;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlValue;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
@XmlRootElement(name="user")
@XmlAccessorType(XmlAccessType.NONE)
public final class UserInfo extends Object {
    @XmlAttribute
    private String id;
    @XmlElement(name="property")
    private List<Property> properties;

    UserInfo() {
    }

    public UserInfo(String id) {
        this.id = id;
    }

    public String getProperty(String name) {
        if (properties == null) {
            return null;
        }
        for (Property p : properties) {
            if (p.name.equals(name)) {
                return p.value;
            }
        }
        return null;
    }

    public String getId() {
        return id;
    }

    public static final class Property {
        @XmlAttribute
        private String name;
        @XmlValue
        private String value;

        private Property() {
        }

        Property(String name, String value) {
            this.name = name;
            this.value = value;
        }
    }
}
