/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */
package cz.xelfi.quoridor.freemarkerdor;

import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.net.httpserver.HttpServer;
import cz.xelfi.quoridor.webidor.resources.Quoridor;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URI;
import java.util.Locale;
import java.util.concurrent.Callable;
import java.util.regex.Pattern;
import javax.ws.rs.core.Cookie;
import javax.ws.rs.core.MediaType;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.netbeans.junit.MockServices;
import static org.junit.Assert.*;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public class ChangeEmailTest extends Object {
    private static File dir;
    private static HttpServer stopAPI;
    private static HttpServer stopStatistics;
    private static Callable<Void> stop;
    private WebResource webResource;
    private WebResource apiResource;
    private WebResource statResource;
    private Client client;

    public ChangeEmailTest() throws Exception {
    }

    @BeforeClass
    public static void localeEnglish() throws Exception {
        Locale.setDefault(Locale.ENGLISH);
        dir = File.createTempFile("quoridor", ".dir");
        dir.delete();
        dir.mkdirs();
        System.setProperty("quoridor.dir", dir.getPath());
        stopAPI = Quoridor.start(7990);
        stopStatistics = Quoridor.start(7992);
        stop = UI.startServers(7991, "http://localhost:7990", "http://localhost:7992", null);

        File passwd = new File(dir, "passwd");
        FileOutputStream os = new FileOutputStream(passwd);
        os.write("test=pes\nJarda=darda\n".getBytes("UTF-8"));
        os.close();

        MockServices.setServices(MockEmailer.class);
    }

    @Before
    public void setUp() throws Exception {
        MockEmailer.clear();

        client = new Client();
        webResource = client.resource(new URI("http://localhost:7991/"));
        apiResource = client.resource(new URI("http://localhost:7990/"));
        statResource = client.resource(new URI("http://localhost:7992/"));
    }

    @AfterClass
    public static void cleanUpAll() throws Exception {
        deleteRec(dir);
        if (stop != null) {
            stop.call();
        }
        if (stopAPI != null) {
            stopAPI.stop(0);
        }
        MockServices.setServices();
    }

    static void deleteRec(File dir) throws IOException {
        if (dir == null) {
            return;
        }
        File[] arr = dir.listFiles();
        if (arr != null) {
            for (File f : arr) {
                deleteRec(f);
            }
        }
        dir.delete();
    }

    @Test public void testChangeEmail() throws Exception {
        String logTest = apiResource.path("login").
            queryParam("name", "test").
            queryParam("password", "pes").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        ClientResponse res = webResource.path("options").
            queryParam("email", "xxx@xxx.cz").
            cookie(Cookie.valueOf("login=" + logTest)).
            accept("text/html").
            get(ClientResponse.class);
        final String txt = res.getEntity(String.class);

        assertEquals("Emails sent to", "xxx@xxx.cz", MockEmailer.address);

        int urlIndex = MockEmailer.text.indexOf(webResource.getURI().toString());
        assertTrue("Found in " + MockEmailer.text, urlIndex >= 0);

        int endIndex = urlIndex;
        while (!Character.isWhitespace(MockEmailer.text.charAt(endIndex))) {
            endIndex++;
        }
        final String url = MockEmailer.text.substring(urlIndex, endIndex);
        WebResource requestURL = client.resource(url);

        String response = requestURL.accept("text/html").get(String.class);
        Pattern p = Pattern.compile("<input *type=\"text\" *name=\"email\" *value='xxx@xxx.cz'");
        assertTrue("New email found" + response, p.matcher(response).find());
    }

    public static final class MockEmailer extends EmailService {
        static String address;
        static String subject;
        static String text;

        @Override
        public void sendEmail(String address, String subject, String text) throws IOException {
            assertNull("No email sent yet", MockEmailer.address);
            MockEmailer.address = address;
            MockEmailer.subject = subject;
            MockEmailer.text = text;
        }

        public static void clear() {
            MockEmailer.address = null;
            MockEmailer.subject = null;
            MockEmailer.text = null;
        }
    }
}
